
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.010.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "A conveyor belt which transports data from a data source to a data sink.",
  "dist_author" => [
    "Matthias Vandermaesen <matthias.vandermaesen\@vlaamsekunstcollectie.be>"
  ],
  "dist_name" => "Datahub-Factory",
  "dist_version" => "1.71",
  "license" => "gpl",
  "module_name" => "Datahub::Factory",
  "recursive_test_files" => 1,
  "requires" => {
    "App::Cmd" => 0,
    "Catmandu" => "1.0603",
    "Catmandu::Importer::XML" => 0,
    "Catmandu::LIDO" => 0,
    "Catmandu::OAI" => 0,
    "Catmandu::Solr" => 0,
    "Catmandu::Store::Datahub" => 0,
    "Config::Simple" => 0,
    "DateTime" => 0,
    "Log::Any" => 0,
    "Log::Any::Adapter" => 0,
    "Log::Log4perl" => 0,
    "Module::Load" => 0,
    "Moo" => 0,
    "MooX::Aliases" => 0,
    "MooX::Role::Logger" => 0,
    "Moose::Role" => 0,
    "Ref::Util" => 0,
    "Sub::Exporter" => 0,
    "Term::ANSIColor" => 0,
    "Try::Tiny" => 0,
    "namespace::clean" => 0,
    "perl" => "5.008005"
  },
  "script_files" => [
    "bin/dhconveyor"
  ],
  "test_requires" => {
    "Log::Any::Adapter::Log4perl" => 0,
    "Log::Any::Test" => "1.03",
    "Log::Log4perl" => 0,
    "Test::Deep" => "0.112",
    "Test::Exception" => "0.43",
    "Test::LWP::UserAgent" => 0,
    "Test::More" => "0.96",
    "Test::Pod" => 0
  }
);


my %fallback_build_requires = (
  "Log::Any::Adapter::Log4perl" => 0,
  "Log::Any::Test" => "1.03",
  "Log::Log4perl" => 0,
  "Module::Build" => "0.28",
  "Test::Deep" => "0.112",
  "Test::Exception" => "0.43",
  "Test::LWP::UserAgent" => 0,
  "Test::More" => "0.96",
  "Test::Pod" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
